from dash import html, Dash, callback, Output, Input, State, no_update
import dash_bootstrap_components as dbc
from dsa_helpers.dash.header import get_header
from os import getenv
from dash_ag_grid import AgGrid

table = AgGrid(
    id="table",
    columnDefs=[],
    rowData=[],
)

app = Dash(
    __name__,
    external_stylesheets=[
        dbc.themes.BOOTSTRAP,
    ],
    title="Test Application",
)

app.layout = html.Div(
    [
        get_header(
            getenv("DSA_API_URL"),
            title="Test Application",
            store_id="user-store",
        ),
        dbc.Row(
            [dbc.Col(dbc.Button("Get items", id="get-item-btn"), width="auto")],
            justify="start",
        ),
        table
    ]
)


@callback(
    [
        Output("table", "columnDefs"),
        Output("table", "rowData"),
    ],
    Input("get-item-btn", "n_clicks"),
    prevent_initial_call=True,
)
def get_items(n_clicks):
    """Get the items."""
    
    

if __name__ == "__main__":
    app.run(
        host="0.0.0.0",
        port=8060,
        debug=True,
    )



# Shadow imports.
from .imread import imread
from .imwrite import imwrite
from . import girder_utils, image_utils, dash, ml, mongo_utils, utils

# Modules that should be available.
__all__ = [
    "girder_utils",
    "image_utils",
    "dash",
    "ml",
    "imread",
    "imwrite",
    "mongo_utils",
    "utils",
]



# Basic utility functions.
import numpy as np


def binary_dice_coefficient(mask1: np.ndarray, mask2: np.ndarray) -> float:
    """Calculate the dice coefficient between two binary masks.

    Args:
        mask1 (np.ndarray): A binary mask.
        mask2 (np.ndarray): A binary mask.

    Returns:
        float: The dice coefficient between the two masks.

    """
    # Convert the masks to binary.
    mask1 = mask1 > 0
    mask2 = mask2 > 0

    # Calculate dice.
    intersection = np.sum(mask1 * mask2)

    sum_masks = np.sum(mask1) + np.sum(mask2)

    if sum_masks == 0:
        return 1.0  # Both masks are empty

    return 2.0 * intersection / sum_masks



from torch.utils.data import Dataset
import pandas as pd
from PIL import Image
import numpy as np


class SegFormerSegmentationDataset(Dataset):
    """PyTorch dataset class for semantic segmentation using HuggingFaces SegFormer
    model.

    """

    def __init__(self, df: pd.DataFrame):
        """Initiate an instance of the segmentation dataset.

        Args:
            df (pandas.DataFrame): A pandas DataFrame with columns "fp" and "mask_fp".

        """
        self.image_files = df["fp"].tolist()
        self.mask_files = df["mask_fp"].tolist()
        self.df = df  # store potential metadata

    def __len__(self):
        return len(self.image_files)

    def __getitem__(self, idx):
        # Get the filepath to the image and its mask.
        image_path = self.image_files[idx]
        mask_path = self.mask_files[idx]

        # Return image & mask as PIL images in a dictionary.
        return {"pixel_values": Image.open(image_path), "label": Image.open(mask_path)}
